from django import forms
from django.conf import settings
from django.http import Http404
from django.utils.translation import ugettext_lazy as _
from dpaste.models import Snippet
from captcha.fields import CaptchaField
from tracking.models import BannedIP
from dpaste.highlight import LEXER_LIST_ALL, LEXER_LIST, LEXER_DEFAULT
import datetime

#===============================================================================
# Snippet Form and Handling
#===============================================================================

EXPIRE_CHOICES = (
    (3600, _(u'In one hour')),
    (10800, _(u'In three hours')),
    (43200, _(u'In twelve hours')),
    (86400, _(u'In one day')),
)

EXPIRE_DEFAULT = 3600

class SnippetCaptchaForm(forms.Form):
    captcha = CaptchaField()

class SnippetForm(forms.ModelForm):

    lexer = forms.ChoiceField(
        choices=LEXER_LIST_ALL,
        initial=LEXER_DEFAULT,
        label=_(u'Lexer'),
    )
    
    expire_options = forms.ChoiceField(
        choices=EXPIRE_CHOICES,
        initial=EXPIRE_DEFAULT,
        label=_(u'Expires'),
    )

    fill_me_out = forms.CharField(required=False)

    def __init__(self, request, *args, **kwargs):
        super(SnippetForm, self).__init__(*args, **kwargs)
        self.request = request
        
        try:
            if self.request.session['userprefs'].get('display_all_lexer', False):
                self.fields['lexer'].choices = LEXER_LIST_ALL
        except KeyError:
            pass

        try:
            self.fields['author'].initial = self.request.session['userprefs'].get('default_name', '')
        except KeyError:
            pass

        try:
            self.fields['expire_options'].initial = self.request.session['userprefs'].get('expire_options', EXPIRE_DEFAULT)
        except KeyError:
            pass


        self.fields['content'].widget=forms.Textarea(attrs={'rows':'20', 'cols':'80'})
        
    def save(self, parent=None, *args, **kwargs):

        # Record IP
        self.instance.ip = self.request.META["REMOTE_ADDR"]

        # Ban and 404 anything that fills this out:
        if self.cleaned_data['fill_me_out'] != '':
            ban_user = BannedIP()
            ban_user.ip_address = self.request.META["REMOTE_ADDR"]
            ban_user.save()
            raise Http404

        # Set parent snippet
        if parent:
            self.instance.parent = parent
        
        # Add expire datestamp
        self.instance.expires = datetime.datetime.now() + \
            datetime.timedelta(seconds=int(self.cleaned_data['expire_options']))

        
        # Save snippet in the db
        super(SnippetForm, self).save(*args, **kwargs)

        # Add the snippet to the user session list
        if self.request.session.get('snippet_list', False):
            if len(self.request.session['snippet_list']) >= getattr(settings, 'MAX_SNIPPETS_PER_USER', 10):
                self.request.session['snippet_list'].pop(0)
            self.request.session['snippet_list'] += [self.instance.pk]
        else:
            self.request.session['snippet_list'] = [self.instance.pk]

        return self.request, self.instance

    class Meta:
        model = Snippet
        fields = (
            'title',
            'content',
            'author',
            'lexer',
        )


#===============================================================================
# User Settings
#===============================================================================

USERPREFS_FONT_CHOICES = [(None, _(u'Default'))] + [
    (i, i) for i in sorted((
        'Monaco',
        'Bitstream Vera Sans Mono',
        'Courier New',
        'Consolas',
    ))
]

USERPREFS_SIZES = [(None, _(u'Default'))] + [(i, '%dpx' % i) for i in range(5, 25)]

class UserSettingsForm(forms.Form):

    default_name = forms.CharField(label=_(u'Default Name'), required=False)
    expire_options = forms.ChoiceField(
        choices=EXPIRE_CHOICES,
        initial=EXPIRE_DEFAULT,
        label=_(u'Expires'),
    )
    #display_all_lexer = forms.BooleanField(
    #    label=_(u'Display all lexer'), 
    #    required=False,
    #    widget=forms.CheckboxInput,
    #    help_text=_(u'This also enables the super secret \'guess lexer\' function.'),
    #)
    #font_family = forms.ChoiceField(label=_(u'Font Family'), required=False, choices=USERPREFS_FONT_CHOICES)
    #font_size = forms.ChoiceField(label=_(u'Font Size'), required=False, choices=USERPREFS_SIZES)
    #line_height = forms.ChoiceField(label=_(u'Line Height'), required=False, choices=USERPREFS_SIZES)
